package ssw.mj.test;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;
import ssw.mj.Messages;

/**
 * Allows testing of output.
 * 
 * @author AW
 */
public class TestPrintWriter extends PrintWriter {
	private List actualLines;
	private List expectedLines;
	
	private StringBuffer curLine;
	
	TestPrintWriter () { 
		super(System.out, true);
		actualLines = new ArrayList();
		expectedLines = new ArrayList();
		curLine = new StringBuffer();
	}
	
	public void print (String s) {
		super.print(s);
		curLine.append(s);
		
		// print a stack trace
		// filter out all methods that are not part of the compiler implementation
		super.print("  || ");
		
		StackTraceElement[] st = (new Throwable()).getStackTrace();
		
		for (int i = 0; i < st.length; i++) {
			String className = st[i].getClassName();
			if (className.startsWith("ssw.mj") && !className.startsWith("ssw.mj.test")) {
				String line = st[i].toString();
				// only show method name and source location
				int pos = line.indexOf('(');
				if (pos >= 0) pos = line.lastIndexOf('.', pos);
				if (pos < 0) pos = 0;
				else pos++;
				
				super.print(" << " + line.substring(pos));
			}
        }
	}

	public void println (String s) {
		// print the string
		print(s);
		super.println();
		
		// start a new line
		actualLines.add(curLine.toString());
		curLine = new StringBuffer();
	}

	void addExpectedLine (String line) { expectedLines.add(line); }
	void addExpectedLine (String prefix, String msgKey) {
		addExpectedLine(prefix, msgKey, "");
	}
	void addExpectedLine (String prefix, String msgKey, String postfix) {
		StringBuffer msg = new StringBuffer();
		msg.append(prefix);
		msg.append(Messages.getString(msgKey));
		msg.append(postfix);
		expectedLines.add(msg.toString());
	}
	
	void verify () {
		// check number of lines
		if (actualLines.size() != expectedLines.size())
			Assert.fail("Expected " + expectedLines.size() + 
			            " output lines, but encountered " +	actualLines.size() + 
			            "\n" + printActualLines());
		// compare lines
		for (int i = 0; i < actualLines.size(); i++) {
			String expectedLine = (String) expectedLines.get(i);
			String actualLine = (String) actualLines.get(i);
			Assert.assertEquals(expectedLine, actualLine);	
		}
	}
	
	private String printActualLines () {
		StringBuffer sb = new StringBuffer();
		
		Iterator iter = actualLines.iterator();
		while (iter.hasNext()) sb.append((String) iter.next() + "\n");
		
		return sb.toString();
	}
}
